@@ -26,9 +26,10 @@
 #include "gal/util/e-util.h"
 #include <X11/Xlib.h>
 #include <gtk/gtkmain.h>
+#include <gtk/gtkimmulticontext.h>
 
 static void e_canvas_init           (ECanvas         *card);
-static void e_canvas_destroy        (GtkObject        *object);
+static void e_canvas_dispose        (GObject        *object);
 static void e_canvas_class_init	    (ECanvasClass    *klass);
 static void e_canvas_realize        (GtkWidget        *widget);
 static void e_canvas_unrealize      (GtkWidget        *widget);
@@ -51,6 +52,7 @@ static void e_canvas_style_set      (GtkWidget        *widget,
 
 static int emit_event (GnomeCanvas *canvas, GdkEvent *event);
 
+#define PARENT_TYPE GNOME_TYPE_CANVAS
 static GnomeCanvasClass *parent_class = NULL;
 
 #define d(x)
@@ -62,45 +64,27 @@ enum {
 
 static guint e_canvas_signals [LAST_SIGNAL] = { 0, };
 
-GtkType
-e_canvas_get_type (void)
-{
-	static GtkType canvas_type = 0;
-
-	if (!canvas_type)
-	{
-		static const GtkTypeInfo canvas_info =
-		{
-			"ECanvas",
-			sizeof (ECanvas),
-			sizeof (ECanvasClass),
-			(GtkClassInitFunc) e_canvas_class_init,
-			(GtkObjectInitFunc) e_canvas_init,
-			/* reserved_1 */ NULL,
-			/* reserved_2 */ NULL,
-			(GtkClassInitFunc) NULL,
-		};
-
-		canvas_type = gtk_type_unique (gnome_canvas_get_type (), &canvas_info);
-	}
-
-	return canvas_type;
-}
+E_MAKE_TYPE (e_canvas,
+	     "ECanvas",
+	     ECanvas,
+	     e_canvas_class_init,
+	     e_canvas_init,
+	     PARENT_TYPE)
 
 static void
 e_canvas_class_init (ECanvasClass *klass)
 {
-	GtkObjectClass *object_class;
+	GObjectClass *object_class;
 	GnomeCanvasClass *canvas_class;
 	GtkWidgetClass *widget_class;
 
-	object_class                       = (GtkObjectClass*) klass;
+	object_class                       = (GObjectClass*) klass;
 	canvas_class                       = (GnomeCanvasClass *) klass;
 	widget_class                       = (GtkWidgetClass *) klass;
 
-	parent_class                       = gtk_type_class (gnome_canvas_get_type ());
+	parent_class                       = g_type_class_ref (PARENT_TYPE);
 
-	object_class->destroy              = e_canvas_destroy;
+	object_class->dispose              = e_canvas_dispose;
 
 	widget_class->key_press_event      = e_canvas_key;
 	widget_class->key_release_event    = e_canvas_key;
@@ -115,14 +99,13 @@ e_canvas_class_init (ECanvasClass *klass)
 	klass->reflow                      = NULL;
 
 	e_canvas_signals [REFLOW] =
-		gtk_signal_new ("reflow",
-				GTK_RUN_LAST,
-				E_OBJECT_CLASS_TYPE (object_class),
-				GTK_SIGNAL_OFFSET (ECanvasClass, reflow),
-				gtk_marshal_NONE__NONE,
-				GTK_TYPE_NONE, 0);
-
-	E_OBJECT_CLASS_ADD_SIGNALS (object_class, e_canvas_signals, LAST_SIGNAL);
+		g_signal_new ("reflow",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (ECanvasClass, reflow),
+			      NULL, NULL,
+			      e_marshal_NONE__NONE,
+			      G_TYPE_NONE, 0);
 }
 
 static void
@@ -130,13 +113,12 @@ e_canvas_init (ECanvas *canvas)
 {
 	canvas->selection = NULL;
 	canvas->cursor = NULL;
-	canvas->ic = NULL;
-	canvas->ic_attr = NULL;
+	canvas->im_context = gtk_im_multicontext_new ();
 	canvas->tooltip_window = NULL;
 }
 
 static void
-e_canvas_destroy (GtkObject *object)
+e_canvas_dispose (GObject *object)
 {
 	ECanvas *canvas = E_CANVAS(object);
 
@@ -144,26 +126,30 @@ e_canvas_destroy (GtkObject *object)
 		g_source_remove(canvas->idle_id);
 	canvas->idle_id = 0;
 
+	if (canvas->grab_cancelled_check_id)
+		g_source_remove (canvas->grab_cancelled_check_id);
+	canvas->grab_cancelled_check_id = 0;
+
 	if (canvas->toplevel) {
 		if (canvas->visibility_notify_id)
-			gtk_signal_disconnect (GTK_OBJECT(canvas->toplevel),
-					       canvas->visibility_notify_id);
+			g_signal_handler_disconnect (canvas->toplevel,
+						     canvas->visibility_notify_id);
 		canvas->visibility_notify_id = 0;
 
-		gtk_object_unref(GTK_OBJECT(canvas->toplevel));
+		g_object_unref (canvas->toplevel);
 		canvas->toplevel = NULL;
 	}
 
 	e_canvas_hide_tooltip(canvas);
 
-	if ((GTK_OBJECT_CLASS (parent_class))->destroy)
-		(*(GTK_OBJECT_CLASS (parent_class))->destroy) (object);
+	if ((G_OBJECT_CLASS (parent_class))->dispose)
+		(*(G_OBJECT_CLASS (parent_class))->dispose) (object);
 }
 
 GtkWidget *
 e_canvas_new ()
 {
-	return GTK_WIDGET (gtk_type_new (e_canvas_get_type ()));
+	return GTK_WIDGET (g_object_new (E_CANVAS_TYPE, NULL));
 }
 
 
@@ -173,7 +159,7 @@ e_canvas_new ()
 static int
 emit_event (GnomeCanvas *canvas, GdkEvent *event)
 {
-	GdkEvent ev;
+	GdkEvent *ev;
 	gint finished;
 	GnomeCanvasItem *item;
 	GnomeCanvasItem *parent;
@@ -237,14 +223,14 @@ emit_event (GnomeCanvas *canvas, GdkEvent *event)
 	 * offsets of the fields in the event structures.
 	 */
 
-	ev = *event;
+	ev = gdk_event_copy (event);
 
-	switch (ev.type) {
+	switch (ev->type) {
 	case GDK_ENTER_NOTIFY:
 	case GDK_LEAVE_NOTIFY:
 		gnome_canvas_window_to_world (canvas,
-					      ev.crossing.x, ev.crossing.y,
-					      &ev.crossing.x, &ev.crossing.y);
+					      ev->crossing.x, ev->crossing.y,
+					      &ev->crossing.x, &ev->crossing.y);
 		break;
 
 	case GDK_MOTION_NOTIFY:
@@ -253,8 +239,8 @@ emit_event (GnomeCanvas *canvas, GdkEvent *event)
 	case GDK_3BUTTON_PRESS:
 	case GDK_BUTTON_RELEASE:
 		gnome_canvas_window_to_world (canvas,
-					      ev.motion.x, ev.motion.y,
-					      &ev.motion.x, &ev.motion.y);
+					      ev->motion.x, ev->motion.y,
+					      &ev->motion.x, &ev->motion.y);
 		break;
 
 	default:
@@ -269,21 +255,18 @@ emit_event (GnomeCanvas *canvas, GdkEvent *event)
 	finished = FALSE;
 
 	while (item && !finished) {
-		gtk_object_ref (GTK_OBJECT (item));
+		g_object_ref (item);
 
-		gtk_signal_emit_by_name (GTK_OBJECT (item), "event",
-					 &ev,
-					 &finished);
-
-		if (GTK_OBJECT_DESTROYED (item))
-			finished = TRUE;
+		g_signal_emit_by_name (item, "event", ev, &finished);
 
 		parent = item->parent;
-		gtk_object_unref (GTK_OBJECT (item));
+		g_object_unref (item);
 
 		item = parent;
 	}
 
+	gdk_event_free (ev);
+
 	return finished;
 }
 
@@ -333,7 +316,7 @@ gnome_canvas_item_invoke_point (GnomeCanvasItem *item, double x, double y, int c
 	y = i.y;
 #endif
 
-	return (* GNOME_CANVAS_ITEM_CLASS (item->object.klass)->point) (
+	return (* GNOME_CANVAS_ITEM_CLASS (GTK_OBJECT_GET_CLASS (item))->point) (
 		item, x, y, cx, cy, actual_item);
 }
 
@@ -409,11 +392,11 @@ pick_current_item (GnomeCanvas *canvas, GdkEvent *event)
 		/* these fields don't have the same offsets in both types of events */
 
 		if (canvas->pick_event.type == GDK_ENTER_NOTIFY) {
-			x = canvas->pick_event.crossing.x + DISPLAY_X1 (canvas) - canvas->zoom_xofs;
-			y = canvas->pick_event.crossing.y + DISPLAY_Y1 (canvas) - canvas->zoom_yofs;
+			x = canvas->pick_event.crossing.x + canvas->scroll_x1 - canvas->zoom_xofs;
+			y = canvas->pick_event.crossing.y + canvas->scroll_y1 - canvas->zoom_yofs;
 		} else {
-			x = canvas->pick_event.motion.x + DISPLAY_X1 (canvas) - canvas->zoom_xofs;
-			y = canvas->pick_event.motion.y + DISPLAY_Y1 (canvas) - canvas->zoom_yofs;
+			x = canvas->pick_event.motion.x + canvas->scroll_x1 - canvas->zoom_xofs;
+			y = canvas->pick_event.motion.y + canvas->scroll_y1 - canvas->zoom_yofs;
 		}
 
 		/* canvas pixel coords */
@@ -635,8 +618,7 @@ e_canvas_focus_in (GtkWidget *widget, GdkEventFocus *event)
 
 	GTK_WIDGET_SET_FLAGS (widget, GTK_HAS_FOCUS);
 
-	if (ecanvas->ic)
-		gdk_im_begin (ecanvas->ic, canvas->layout.bin_window);
+	gtk_im_context_focus_in (ecanvas->im_context);
 
 	if (canvas->focused_item) {
 		full_event.focus_change = *event;
@@ -659,8 +641,7 @@ e_canvas_focus_out (GtkWidget *widget, GdkEventFocus *event)
 
 	GTK_WIDGET_UNSET_FLAGS (widget, GTK_HAS_FOCUS);
 
-	if (ecanvas->ic)
-		gdk_im_end ();
+	gtk_im_context_focus_out (ecanvas->im_context);
 
 	if (canvas->focused_item) {
 		full_event.focus_change = *event;
@@ -673,13 +654,13 @@ e_canvas_focus_out (GtkWidget *widget, GdkEventFocus *event)
 static void
 ec_style_set_recursive (GnomeCanvasItem *item, GtkStyle *previous_style)
 {
-	guint signal_id = gtk_signal_lookup ("style_set", GTK_OBJECT_TYPE (item));
+	guint signal_id = g_signal_lookup ("style_set", G_OBJECT_TYPE (item));
 	if (signal_id >= 1) {
-		GtkSignalQuery *query = gtk_signal_query (signal_id);
-		if (query->return_val == GTK_TYPE_NONE && query->nparams == 1 && query->params[0] == GTK_TYPE_STYLE) {
-			gtk_signal_emit (GTK_OBJECT (item), signal_id, previous_style);
+		GSignalQuery query;
+		g_signal_query (signal_id, &query);
+		if (query.return_type == GTK_TYPE_NONE && query.n_params == 1 && query.param_types[0] == GTK_TYPE_STYLE) {
+			g_signal_emit (item, signal_id, 0, previous_style);
 		}
-		g_free (query);
 	}
 
 	if (GNOME_IS_CANVAS_GROUP (item) ) {
@@ -699,7 +680,6 @@ e_canvas_style_set (GtkWidget *widget, GtkStyle *previous_style)
 static void
 e_canvas_realize (GtkWidget *widget)
 {
-	gint width, height;
 	ECanvas *ecanvas = E_CANVAS (widget);
 
 	if (GTK_WIDGET_CLASS (parent_class)->realize)
@@ -707,60 +687,7 @@ e_canvas_realize (GtkWidget *widget)
 
 	gdk_window_set_back_pixmap (GTK_LAYOUT (widget)->bin_window, NULL, FALSE);
 
-	if (gdk_im_ready () && (ecanvas->ic_attr = gdk_ic_attr_new ()) != NULL) {
-		GdkEventMask mask;
-		GdkICAttr *attr = ecanvas->ic_attr;
-		GdkICAttributesType attrmask = GDK_IC_ALL_REQ;
-		GdkIMStyle style;
-		GdkIMStyle supported_style = GDK_IM_PREEDIT_NONE |
-			GDK_IM_PREEDIT_NOTHING |
-			GDK_IM_PREEDIT_POSITION |
-			GDK_IM_STATUS_NONE |
-			GDK_IM_STATUS_NOTHING;
-
-		if(widget->style && widget->style->font->type != GDK_FONT_FONTSET)
-			supported_style &= ~GDK_IM_PREEDIT_POSITION;
-
-		attr->style = style = gdk_im_decide_style (supported_style);
-		attr->client_window = ecanvas->parent.layout.bin_window;
-
-		switch (style & GDK_IM_PREEDIT_MASK)
-			{
-			case GDK_IM_PREEDIT_POSITION:
-				if (widget->style && widget->style->font->type != GDK_FONT_FONTSET)
-					{
-						g_warning ("over-the-spot style requires fontset");
-						break;
-					}
-				
-				gdk_window_get_size (attr->client_window, &width, &height);
-				height = widget->style->font->ascent +
-					widget->style->font->descent;
-
-				attrmask |= GDK_IC_PREEDIT_POSITION_REQ;
-				attr->spot_location.x = 0;
-				attr->spot_location.y = height;
-				attr->preedit_area.x = 0;
-				attr->preedit_area.y = 0;
-				attr->preedit_area.width = width;
-				attr->preedit_area.height = height;
-				attr->preedit_fontset = widget->style->font;
-
-				break;
-			}
-
-		ecanvas->ic = gdk_ic_new (attr, attrmask);
-		if (ecanvas->ic != NULL) {
-			mask = gdk_window_get_events (attr->client_window);
-			mask |= gdk_ic_get_events (ecanvas->ic);
-			gdk_window_set_events (attr->client_window, mask);
-
-			if (GTK_WIDGET_HAS_FOCUS (widget))
-				gdk_im_begin (ecanvas->ic, attr->client_window);
-		} else
-			g_warning ("Can't create input context.");
-	}
-
+	gtk_im_context_set_client_window (ecanvas->im_context, widget->window);
 }
 
 static void
@@ -768,18 +695,13 @@ e_canvas_unrealize (GtkWidget *widget)
 {
 	ECanvas * ecanvas = E_CANVAS (widget);
 
-	if (ecanvas->idle_id)
+	if (ecanvas->idle_id) {
 		g_source_remove(ecanvas->idle_id);
-	ecanvas->idle_id = 0;
+		ecanvas->idle_id = 0;
+	}
+
+	gtk_im_context_set_client_window (ecanvas->im_context, widget->window);
 
-	if (ecanvas->ic) {
-		gdk_ic_destroy (ecanvas->ic);
-		ecanvas->ic = NULL;
-	}
-	if (ecanvas->ic_attr) {
-		gdk_ic_attr_destroy (ecanvas->ic_attr);
-		ecanvas->ic_attr = NULL;
-	}
 	if (GTK_WIDGET_CLASS (parent_class)->unrealize)
 		(* GTK_WIDGET_CLASS (parent_class)->unrealize) (widget);
 }
@@ -803,8 +725,8 @@ e_canvas_item_invoke_reflow (GnomeCanvasItem *item, int flags)
 	if (item->object.flags & E_CANVAS_ITEM_NEEDS_REFLOW) {
 		ECanvasItemReflowFunc func;
 		func = (ECanvasItemReflowFunc)
-			gtk_object_get_data (GTK_OBJECT (item),
-					     "ECanvasItem::reflow_callback");
+			g_object_get_data (G_OBJECT (item),
+					   "ECanvasItem::reflow_callback");
 		if (func)
 			func (item, flags);
 	}
@@ -834,8 +756,8 @@ idle_handler (gpointer data)
 	/* Reset idle id */
 	canvas->idle_id = 0;
 
-	gtk_signal_emit (GTK_OBJECT (canvas),
-			 e_canvas_signals [REFLOW]);
+	g_signal_emit (canvas,
+		       e_canvas_signals [REFLOW], 0);
 
 	GDK_THREADS_LEAVE();
 
@@ -884,20 +806,20 @@ e_canvas_item_request_parent_reflow (GnomeCanvasItem *item)
 void
 e_canvas_item_set_reflow_callback (GnomeCanvasItem *item, ECanvasItemReflowFunc func)
 {
-	gtk_object_set_data(GTK_OBJECT(item), "ECanvasItem::reflow_callback", (gpointer) func);
+	g_object_set_data(G_OBJECT(item), "ECanvasItem::reflow_callback", (gpointer) func);
 }
 
 
 void
 e_canvas_item_set_selection_callback (GnomeCanvasItem *item, ECanvasItemSelectionFunc func)
 {
-	gtk_object_set_data(GTK_OBJECT(item), "ECanvasItem::selection_callback", (gpointer) func);
+	g_object_set_data(G_OBJECT(item), "ECanvasItem::selection_callback", (gpointer) func);
 }
 
 void
 e_canvas_item_set_selection_compare_callback (GnomeCanvasItem *item, ECanvasItemSelectionCompareFunc func)
 {
-	gtk_object_set_data(GTK_OBJECT(item), "ECanvasItem::selection_compare_callback", (gpointer) func);
+	g_object_set_data(G_OBJECT(item), "ECanvasItem::selection_compare_callback", (gpointer) func);
 }
 
 void
@@ -920,12 +842,13 @@ e_canvas_item_set_cursor (GnomeCanvasItem *item, gpointer id)
 	for (list = canvas->selection; list; list = g_list_next(list)) {
 		info = list->data;
 
-		func = (ECanvasItemSelectionFunc)gtk_object_get_data(GTK_OBJECT(info->item), "ECanvasItem::selection_callback");
+		func = (ECanvasItemSelectionFunc)g_object_get_data(G_OBJECT(info->item),
+								   "ECanvasItem::selection_callback");
 		if (func)
 			func(info->item, flags, info->id);
 		g_message ("ECANVAS: free info (2): item %p, id %p",
 			   info->item, info->id);
-		gtk_object_unref (GTK_OBJECT (info->item));
+		g_object_unref (info->item);
 		g_free(info);
 	}
 	g_list_free(canvas->selection);
@@ -936,12 +859,13 @@ e_canvas_item_set_cursor (GnomeCanvasItem *item, gpointer id)
 
 	info = g_new(ECanvasSelectionInfo, 1);
 	info->item = item;
-	gtk_object_ref (GTK_OBJECT (info->item));
+	g_object_ref (info->item);
 	info->id = id;
 	g_message ("ECANVAS: new info item %p, id %p", item, id);
 
 	flags = E_CANVAS_ITEM_SELECTION_SELECT | E_CANVAS_ITEM_SELECTION_CURSOR;
-	func = (ECanvasItemSelectionFunc)gtk_object_get_data(GTK_OBJECT(item), "ECanvasItem::selection_callback");
+	func = (ECanvasItemSelectionFunc)g_object_get_data(G_OBJECT(item),
+							   "ECanvasItem::selection_callback");
 	if (func)
 		func(item, flags, id);
 
@@ -972,7 +896,8 @@ e_canvas_item_add_selection (GnomeCanvasItem *item, gpointer id)
 	canvas = E_CANVAS(item->canvas);
 
 	if (canvas->cursor) {
-		func = (ECanvasItemSelectionFunc)gtk_object_get_data(GTK_OBJECT(canvas->cursor->item), "ECanvasItem::selection_callback");
+		func = (ECanvasItemSelectionFunc)g_object_get_data(G_OBJECT(canvas->cursor->item),
+								   "ECanvasItem::selection_callback");
 		if (func)
 			func(canvas->cursor->item, flags, canvas->cursor->id);
 	}
@@ -987,11 +912,13 @@ e_canvas_item_add_selection (GnomeCanvasItem *item, gpointer id)
 
 		if (search->item == item) {
 			ECanvasItemSelectionCompareFunc compare_func;
-			compare_func = (ECanvasItemSelectionCompareFunc)gtk_object_get_data(GTK_OBJECT(search->item), "ECanvasItem::selection_compare_callback");
+			compare_func = (ECanvasItemSelectionCompareFunc)g_object_get_data(G_OBJECT(search->item),
+											  "ECanvasItem::selection_compare_callback");
 
 			if (compare_func(search->item, search->id, id, 0) == 0) {
 				canvas->cursor = search;
-				func = (ECanvasItemSelectionFunc)gtk_object_get_data(GTK_OBJECT(item), "ECanvasItem::selection_callback");
+				func = (ECanvasItemSelectionFunc)g_object_get_data(G_OBJECT(item),
+										   "ECanvasItem::selection_callback");
 				if (func)
 					func(item, flags, search->id);
 				return;
@@ -1001,11 +928,12 @@ e_canvas_item_add_selection (GnomeCanvasItem *item, gpointer id)
 
 	info = g_new(ECanvasSelectionInfo, 1);
 	info->item = item;
-	gtk_object_ref (GTK_OBJECT (info->item));
+	g_object_ref (info->item);
 	info->id = id;
 	g_message ("ECANVAS: new info (2): item %p, id %p", item, id);
 
-	func = (ECanvasItemSelectionFunc)gtk_object_get_data(GTK_OBJECT(item), "ECanvasItem::selection_callback");
+	func = (ECanvasItemSelectionFunc)g_object_get_data(G_OBJECT(item),
+							   "ECanvasItem::selection_callback");
 	if (func)
 		func(item, flags, id);
 
@@ -1034,11 +962,13 @@ e_canvas_item_remove_selection (GnomeCanvasItem *item, gpointer id)
 
 		if (info->item == item) {
 			ECanvasItemSelectionCompareFunc compare_func;
-			compare_func = (ECanvasItemSelectionCompareFunc)gtk_object_get_data(GTK_OBJECT(info->item), "ECanvasItem::selection_compare_callback");
+			compare_func = (ECanvasItemSelectionCompareFunc)g_object_get_data(G_OBJECT(info->item),
+											  "ECanvasItem::selection_compare_callback");
 
 			if (compare_func(info->item, info->id, id, 0) == 0) {
 				ECanvasItemSelectionFunc func;
-				func = (ECanvasItemSelectionFunc) gtk_object_get_data(GTK_OBJECT(info->item), "ECanvasItem::selection_callback");
+				func = (ECanvasItemSelectionFunc) g_object_get_data(G_OBJECT(info->item),
+										    "ECanvasItem::selection_callback");
 				if (func)
 					func(info->item, flags, info->id);
 				canvas->selection = g_list_remove_link(canvas->selection, list);
@@ -1048,7 +978,7 @@ e_canvas_item_remove_selection (GnomeCanvasItem *item, gpointer id)
 
 				g_message ("ECANVAS: removing info: item %p, info %p",
 					   info->item, info->id);
-				gtk_object_unref (GTK_OBJECT (info->item));
+				g_object_unref (info->item);
 				g_free(info);
 				g_list_free_1(list);
 				break;
@@ -1068,13 +998,14 @@ void e_canvas_popup_tooltip (ECanvas *canvas, GtkWidget *widget, int x, int y)
 		canvas->toplevel = gtk_widget_get_toplevel (GTK_WIDGET(canvas));
 		if (canvas->toplevel) {
 			gtk_widget_add_events(canvas->toplevel, GDK_VISIBILITY_NOTIFY_MASK);
-			gtk_object_ref(GTK_OBJECT(canvas->toplevel));
+			g_object_ref (canvas->toplevel);
 			canvas->visibility_notify_id =
-				gtk_signal_connect (GTK_OBJECT (canvas->toplevel), "visibility_notify_event",
-						    GTK_SIGNAL_FUNC (e_canvas_visibility), canvas);
+				g_signal_connect (canvas->toplevel, "visibility_notify_event",
+						  G_CALLBACK (e_canvas_visibility), canvas);
 		}
 	}
-	gtk_widget_popup (widget, x, y);
+	gtk_widget_set_uposition (widget, x, y);
+	gtk_widget_show (widget);
 }
 
 void e_canvas_hide_tooltip  (ECanvas *canvas)
